#include "threadedCam.h"
#include <unistd.h>
using namespace aeroTAP;

aeroTAPLinux3DCamera *aeroCam;

static 	void YUY2_to_RGB24(unsigned char *pSrc, unsigned char *pDst, int cx, int cy);


void YUY2_to_RGB24(unsigned char *pSrc, unsigned char *pDst, int cx, int cy)
{
	int nSrcBPS, nDstBPS, x, y, x2, x3, m;
	int ma0, mb0, m02, m11, m12, m21;
	unsigned char *pS0, *pD0;
	int Y, U, V, Y2;
	unsigned char R, G, B, R2, G2, B2;
	//
	nSrcBPS = cx * 2;
	nDstBPS = ((cx * 3 + 3) / 4) * 4;
	//
	pS0 = pSrc;
#if 1
	pD0 = pDst;
#else
	pD0 = pDst + nDstBPS*(cy - 1);
#endif
	for (y = 0; y<cy; y++) {
		for (x3 = 0, x2 = 0, x = 0; x<cx; x += 2, x2 += 4, x3 += 6) {
			Y = (int)pS0[x2 + 0] - 16;
			Y2 = (int)pS0[x2 + 2] - 16;
			U = (int)pS0[x2 + 1] - 128;
			V = (int)pS0[x2 + 3] - 128;
			//
			ma0 = 298 * Y;
			mb0 = 298 * Y2;
			m02 = 409 * V + 128;
			m11 = -100 * U;
			m12 = -208 * V + 128;
			m21 = 516 * U + 128;
			//
			m = (ma0 + m02) >> 8;
			R = (m<0) ? (0) : (m>255) ? (255) : ((unsigned char)m);
			m = (ma0 + m11 + m12) >> 8;
			G = (m<0) ? (0) : (m>255) ? (255) : ((unsigned char)m);
			m = (ma0 + m21) >> 8;
			B = (m<0) ? (0) : (m>255) ? (255) : ((unsigned char)m);
			//
			m = (mb0 + m02) >> 8;
			R2 = (m<0) ? (0) : (m>255) ? (255) : ((unsigned char)m);
			m = (mb0 + m11 + m12) >> 8;
			G2 = (m<0) ? (0) : (m>255) ? (255) : ((unsigned char)m);
			m = (mb0 + m21) >> 8;
			B2 = (m<0) ? (0) : (m>255) ? (255) : ((unsigned char)m);
			//
#if 1
			pD0[x3 + 2] = B;
			pD0[x3 + 1] = G;
			pD0[x3 + 0] = R;
			pD0[x3 + 5] = B2;
			pD0[x3 + 4] = G2;
			pD0[x3 + 3] = R2;
#else
			pD0[x3] = B;
			pD0[x3 + 1] = G;
			pD0[x3 + 2] = R;
			pD0[x3 + 3] = B2;
			pD0[x3 + 4] = G2;
			pD0[x3 + 5] = R2;
#endif
		}
		pS0 += nSrcBPS;
#if 1
		pD0 += nDstBPS;
#else
		pD0 -= nDstBPS;
#endif
	}
}
threadedCam::threadedCam(){

	interval =33;

	for (int cam = 0; cam <_MAX_CAMERA; ++cam)
	{
		g_nLogIndex[cam] = 0;
		bRunning[cam] = false;

		nDepthHistCnt[cam] = nDepthHistMin[cam] = nDepthHistMax[cam] = nDepthHistCut[cam] = 0;
		memset(nDepthHist[cam], 0, sizeof(int) * 120);

		g_DepthLog[cam] = (WORD*)malloc( sizeof(WORD) * 1280 * 720 * (_LOG + 1));
		memset(g_DepthLog[cam], 0, sizeof(WORD)*(1280 * 720 * (_LOG + 1)));

		g_DepthSum[cam] = (int*)malloc( sizeof(int) * 1280 * 720 );
		memset(g_DepthSum[cam], 0, sizeof(int)*1280 * 720 );

		g_nLogCnt[cam] = (unsigned char*)malloc( 1280 * 720);
		memset(g_nLogCnt[cam], 0, 1280 * 720 );

		camFPS = 30;
		m_nMaxDepth = 3000;
		for (int i = 0; i < 6; ++i)
		{
			m_pInfoHeader[cam][i] = (BITMAPINFOHEADER*)malloc(sizeof(BITMAPINFOHEADER) + 1280 * 720 * 3);
			if (!m_pInfoHeader[cam][i])
			{
				printf("Memory allocation error");
			}
			memset(m_pInfoHeader[cam][i],0, sizeof(BITMAPINFOHEADER));
			m_pInfoHeader[cam][i]->biSize = sizeof(BITMAPINFOHEADER);
			m_pInfoHeader[cam][i]->biWidth = 1280;
			m_pInfoHeader[cam][i]->biHeight = 720;
			m_pInfoHeader[cam][i]->biBitCount = 24;
			m_pInfoHeader[cam][i]->biPlanes = 1;
			m_pInfoHeader[cam][i]->biSizeImage = m_pInfoHeader[cam][i]->biWidth * m_pInfoHeader[cam][i]->biHeight * 3;
			m_pInfoHeader[cam][i]->biCompression = BI_RGB;
		}
	}
	bLog = false;
}

threadedCam::~threadedCam(){

	for (int cam = 0; cam < _MAX_CAMERA; ++cam)
	{
		if (bRunning[cam] )
			stop(cam);
		for (int i = 0; i < 6; ++i)
		{
			if (m_pInfoHeader[cam][i])
				free(m_pInfoHeader[cam][i]);
			m_pInfoHeader[cam][i] = NULL;
		}
		if (g_DepthLog[cam])
			free(g_DepthLog[cam]);
		g_DepthLog[cam] = NULL;

		if (g_DepthSum[cam])
			free(g_DepthSum[cam]);
		g_DepthSum[cam] = NULL;

		if (g_nLogCnt[cam])
			free(g_nLogCnt[cam]);
		g_nLogCnt[cam] = NULL;
//		release();
	}
}

bool threadedCam::start(int nCamNo, bool bStereo){

		m_bStereo = bStereo;
		m_nCamNo = nCamNo;

	depthWidth = camWidth;
	depthHeight = camHeight;
	{
		m_rectView.left = m_rectView.top =0;
		m_rectView.right = depthWidth;
		m_rectView.bottom =depthHeight;

		for(int i=4;i<6;++i)
		{
			m_pInfoHeader[nCamNo][i]->biWidth = camWidth;
			m_pInfoHeader[nCamNo][i]->biHeight = camHeight;
			m_pInfoHeader[nCamNo][i]->biBitCount = 8;
			m_pInfoHeader[nCamNo][i]->biPlanes = 1;
			m_pInfoHeader[nCamNo][i]->biSizeImage = camWidth*camHeight;
			m_pInfoHeader[nCamNo][i]->biCompression = BI_RGB;
		}

		aeroCam = new aeroTAPLinux3DCamera(camWidth, camHeight, camWidth / 2, camHeight);
		loadZDTable(nCamNo);

		printf(".. zdTable %d",nTableLen[nCamNo]);
		if (camWidth == 1280)
			nTableLen[nCamNo] = 0;

		if (bRunning[0] == false && bRunning[1] == false)
			startThread(true);   // blocking
		bRunning[nCamNo] = true;

	}
	return true;
}

void threadedCam::stop(int nCamNo){

	bRunning[nCamNo] = false;

	if (bRunning[0] == false && bRunning[1] == false)
		stopThread();
   sleep( 1);
}

void threadedCam::reset(){
        if(lock()){
            path = "";
            unlock();
        }
}

void threadedCam::setDepthMax(int max){

	for (int cam = 0; cam < _MAX_CAMERA; ++cam)
	{
		m_nMaxDepth = max;
	}
}


void threadedCam::SetFocalLength(float w, float h) {

	nFocalLengthW = w;
	nFocalLengthH = h;
}

int threadedCam::loadZDTable(int cam){
	int nLen = 0;

	string zdName;
	if (camWidth == 320 && camHeight == 240)
		zdName = "zdTable_QVGA.txt";
	else
	if (camWidth == 640 && camHeight == 400)
		zdName = "zdTable_WVGA.txt";
	else
		zdName = "zdTable_VGA.txt";

	printf("Load zdTable %s",ofToDataPath(zdName).c_str());

	ifstream  f;
	f.open(ofToDataPath(zdName).c_str());
	if (!f.fail())
	{
		vector <string> strLines;
		while (!f.eof()) {
			string ptStr;
			getline(f, ptStr);
			if ( ptStr.length()>0 )
			{
				sscanf(ptStr.c_str(), "%d", &zdTable[cam][nLen]);
				++nLen;
			}
		}
		f.close();
	}
	nTableLen[cam] = nLen;

	return nTableLen[cam];
}

unsigned char *threadedCam::getImage(int cam) {

	return (unsigned char*)(m_pInfoHeader[cam][1] + 1);
}

int threadedCam::getType() {
	return nPType;
}

void threadedCam::setLog(bool b){
		bLog = b;
}


unsigned char *threadedCam::getDepth(int cam){

	WORD *pDepthAveW = (WORD*)(&g_DepthLog[cam][1280 * 720 * _LOG  ]);
	return (unsigned char*)pDepthAveW;

}

int threadedCam::GetActualFPS(int cam){
	return 0;
}

bool threadedCam::isFrameNew(int cam){

	bool ret = bNewFrame[cam];
	bNewFrame[cam] = false;
	return ret;
}

int threadedCam::setIR(int nVal) {
	return 0;
}

void threadedCam::threadedFunction(){

 emCamStreamFormat typeC = emCamStreamFormatColor;
 emCamStreamFormat typeD = emCamStreamFormatDepth;
	int nCamNo = _MAX_CAMERA;
    while(isThreadRunning() != 0){
			for (int cam = 0; cam <nCamNo; ++cam)
			{
//				printf("Update Camera..");
				aeroCam->updateFrame();
				{
						if (lock()) {
							unlock();
						}
						memset(nDepthHist[cam],0,sizeof(int)*120);
						nDepthHistCnt[cam] = nDepthHistMin[cam] = nDepthHistMax[cam] = nDepthHistCut[cam] =0;

						unsigned char *pColor = aeroCam->getFrame(typeC);
						unsigned char *pDepth = aeroCam->getFrame(typeD);

						YUY2_to_RGB24(pColor, (unsigned char*)(m_pInfoHeader[cam][1] + 1), camWidth, camHeight);

						WORD *pDepthW = (WORD*)pDepth;
						WORD *pDepthAveW = (WORD*)(&g_DepthLog[cam][1280 * 720 * _LOG]);

//		printf("zdTable %d",nTableLen[cam]);

						if (nTableLen[cam] != 0)
						{
							for (int i = 0; i < camWidth*camHeight; ++i)
							{
								WORD  nDepth = pDepth[i];
								if (nTableLen[cam] != 256)
								{
									if (bUSB2)
										nDepth = (nDepth << 3);
									else
									{
										nDepth = pDepthW[i] & 0x3FF;
									}
								}
								nDepth = zdTable[cam][nDepth];
								pDepthAveW[i] = nDepth;
								//  < 6m
								if (nDepth>0 && nDepth < 120 * 50)
								{
									++nDepthHist[cam][nDepth/50];
									++nDepthHistCnt[cam];
								}

							}

						}
						else
						{
							for (int i = 0; i < camWidth*camHeight; ++i)
							{
								WORD nDepth = pDepthW[i] & 0x3FFF;
								if (nDepth == 4096)
									nDepth = 0;
								pDepthAveW[i] = nDepth;
								if (nDepth>0 && nDepth < 120 * 50)
								{
									++nDepthHist[cam][nDepth / 50];
									++nDepthHistCnt[cam];
								}
							}
						}
						int nTotal = 0;
						if (nDepthHistCnt[cam] > 0)
						{
							for (int i = 0; i < 120; ++i)
							{
								nTotal += nDepthHist[cam][i];
								if (nDepthHistMax[cam] == 0 || nDepthHist[cam][i]>100 )
									nDepthHistMax[cam] = i;
								if (nDepthHist[cam][i] > 500)
								{
									if (nDepthHistMin[cam] == 0)
										nDepthHistMin[cam] = i * 50;
								}
								if (nDepthHistCut[cam] == 0 && nTotal * 100 / nDepthHistCnt[cam] > 20 && nDepthHist[cam][i]<80)
									nDepthHistCut[cam] = i * 50;
							}
							nDepthHistMax[cam] *= 50;
						}
						if (nDepthHistMax[cam] < 1000)
							nDepthHistMax[cam] = nDepthHistMax[cam];
						bNewFrame[cam] = true;
				}

			}
          ofSleepMillis(interval);
   }
}


